@@ -13,12 +13,11 @@ namespace clad {
 
 #include "Compatibility.h"
 #include "DerivativeBuilder.h"
-#include "clad/Differentiator/DiffMode.h"
 
 #include "clang/AST/RecursiveASTVisitor.h"
 #include "clang/AST/StmtVisitor.h"
+#include "clang/Sema/ParsedAttr.h"
 #include "clang/Sema/Sema.h"
-
 #include <array>
 #include <stack>
 #include <unordered_map>
@@ -32,10 +31,14 @@ namespace clad {
   private:
     std::array<clang::Stmt*, 2> data;
     clang::Stmt* m_DerivativeForForwSweep;
+    clang::Stmt* m_ValueForRevSweep;
+
   public:
     StmtDiff(clang::Stmt* orig = nullptr, clang::Stmt* diff = nullptr,
-             clang::Stmt* forwSweepDiff = nullptr)
-        : m_DerivativeForForwSweep(forwSweepDiff) {
+             clang::Stmt* forwSweepDiff = nullptr,
+             clang::Stmt* valueForRevSweep = nullptr)
+        : m_DerivativeForForwSweep(forwSweepDiff),
+          m_ValueForRevSweep(valueForRevSweep) {
       data[1] = orig;
       data[0] = diff;
     }
@@ -48,7 +51,7 @@ namespace clad {
     clang::Expr* getExpr_dx() {
       return llvm::cast_or_null<clang::Expr>(getStmt_dx());
     }
-    
+
     void updateStmt(clang::Stmt* S) { data[1] = S; }
     void updateStmtDx(clang::Stmt* S) { data[0] = S; }
     // Stmt_dx goes first!
@@ -56,6 +59,18 @@ namespace clad {
 
     clang::Stmt* getForwSweepStmt_dx() { return m_DerivativeForForwSweep; }
 
+    clang::Expr* getRevSweepAsExpr() {
+      return llvm::cast_or_null<clang::Expr>(getRevSweepStmt());
+    }
+
+    clang::Stmt* getRevSweepStmt() {
+      /// If there is no specific value for
+      /// the reverse sweep, use Stmt_dx.
+      if (!m_ValueForRevSweep)
+        return data[1];
+      return m_ValueForRevSweep;
+    }
+
     clang::Expr* getForwSweepExpr_dx() {
       return llvm::cast_or_null<clang::Expr>(m_DerivativeForForwSweep);
     }
@@ -63,32 +78,30 @@ namespace clad {
     void setForwSweepStmt_dx(clang::Stmt* S) { m_DerivativeForForwSweep = S; }
   };
 
-  class VarDeclDiff {
+  template <typename T> class DeclDiff {
   private:
-    std::array<clang::VarDecl*, 2> data;
+    std::array<T*, 2> m_data;
 
   public:
-    VarDeclDiff(clang::VarDecl* orig = nullptr,
-                clang::VarDecl* diff = nullptr) {
-      data[1] = orig;
-      data[0] = diff;
+    DeclDiff(T* orig = nullptr, T* diff = nullptr) {
+      m_data[1] = orig;
+      m_data[0] = diff;
     }
 
-    clang::VarDecl* getDecl() { return data[1]; }
-    clang::VarDecl* getDecl_dx() { return data[0]; }
+    T* getDecl() { return m_data[1]; }
+    T* getDecl_dx() { return m_data[0]; }
     // Decl_dx goes first!
-    std::array<clang::VarDecl*, 2>& getBothDecls() { return data; }
+    std::array<T*, 2>& getBothDecls() { return m_data; }
   };
 
   /// A base class for all common functionality for visitors
   class VisitorBase {
   protected:
-    VisitorBase(DerivativeBuilder& builder)
+    VisitorBase(DerivativeBuilder& builder, const DiffRequest& request)
         : m_Builder(builder), m_Sema(builder.m_Sema),
           m_CladPlugin(builder.m_CladPlugin), m_Context(builder.m_Context),
-          m_CurScope(m_Sema.TUScope), m_DerivativeFnScope(nullptr),
-          m_DerivativeInFlight(false), m_Derivative(nullptr),
-          m_Function(nullptr) {}
+          m_DerivativeFnScope(nullptr), m_DerivativeInFlight(false),
+          m_Derivative(nullptr), m_DiffReq(request) {}
 
     using Stmts = llvm::SmallVector<clang::Stmt*, 16>;
 
@@ -97,15 +110,13 @@ namespace clad {
     plugin::CladPlugin& m_CladPlugin;
     clang::ASTContext& m_Context;
     /// Current Scope at the point of visiting.
-    clang::Scope* m_CurScope;
     /// Pointer to the topmost Scope in the created derivative function.
     clang::Scope* m_DerivativeFnScope;
     bool m_DerivativeInFlight;
     /// The Derivative function that is being generated.
     clang::FunctionDecl* m_Derivative;
-    /// The function that is currently differentiated.
-    const clang::FunctionDecl* m_Function;
-    DiffMode m_Mode;
+    /// The differentiation request that is being currently processed.
+    const DiffRequest& m_DiffReq;
     /// Map used to keep track of variable declarations and match them
     /// with their derivatives.
     std::unordered_map<const clang::ValueDecl*, clang::Expr*> m_Variables;
@@ -131,10 +142,12 @@ namespace clad {
     // FIXME: Fix this inconsistency, by making `this` pointer derivative
     // expression to be of object type in the reverse mode as well.
     clang::Expr* m_ThisExprDerivative = nullptr;
+
     /// A function used to wrap result of visiting E in a lambda. Returns a call
     /// to the built lambda. Func is a functor that will be invoked inside
     /// lambda scope and block. Statements inside lambda are expected to be
     /// added by addToCurrentBlock from func invocation.
+    // FIXME: This will become problematic when we try to support C.
     template <typename F>
     static clang::Expr* wrapInLambda(VisitorBase& V, clang::Sema& S,
                                      const clang::Expr* E, F&& func) {
@@ -147,19 +160,41 @@ namespace clad {
       Intro.Range.setBegin(E->getBeginLoc());
       Intro.Range.setEnd(E->getEndLoc());
       clang::AttributeFactory AttrFactory;
-      clang::DeclSpec DS(AttrFactory);
+      const clang::DeclSpec DS(AttrFactory);
       clang::Declarator D(DS,
-                          clad_compat::DeclaratorContext::
-                              CLAD_COMPAT_CLANG12_Declarator_LambdaExpr);
+                          CLAD_COMPAT_CLANG15_Declarator_DeclarationAttrs_ExtraParam
+                          CLAD_COMPAT_CLANG12_Declarator_LambdaExpr);
+#if CLANG_VERSION_MAJOR > 16
+      V.beginScope(clang::Scope::LambdaScope | clang::Scope::DeclScope |
+
+                   clang::Scope::FunctionDeclarationScope |
+                   clang::Scope::FunctionPrototypeScope);
+#endif // CLANG_VERSION_MAJOR
       S.PushLambdaScope();
+#if CLANG_VERSION_MAJOR > 16
+      S.ActOnLambdaExpressionAfterIntroducer(Intro, V.getCurrentScope());
+
+      S.ActOnLambdaClosureParameters(V.getCurrentScope(), /*ParamInfo=*/{});
+#endif // CLANG_VERSION_MAJOR
+
       V.beginScope(clang::Scope::BlockScope | clang::Scope::FnScope |
-                   clang::Scope::DeclScope);
-      S.ActOnStartOfLambdaDefinition(Intro, D, V.getCurrentScope());
+                   clang::Scope::DeclScope | clang::Scope::CompoundStmtScope);
+      S.ActOnStartOfLambdaDefinition(Intro, D,
+                   clad_compat::Sema_ActOnStartOfLambdaDefinition_ScopeOrDeclSpec(V.getCurrentScope(), DS));
+#if CLANG_VERSION_MAJOR > 16
+      V.endScope();
+#endif // CLANG_VERSION_MAJOR
+
       V.beginBlock();
       func();
       clang::CompoundStmt* body = V.endBlock();
       clang::Expr* lambda =
-          S.ActOnLambdaExpr(noLoc, body, V.getCurrentScope()).get();
+          S.ActOnLambdaExpr(
+               noLoc,
+               body /*,*/
+                   CLAD_COMPAT_CLANG17_ActOnLambdaExpr_getCurrentScope_ExtraParam(
+                       V))
+              .get();
       V.endScope();
       return S.ActOnCallExpr(V.getCurrentScope(), lambda, noLoc, {}, noLoc)
           .get();
@@ -193,22 +228,17 @@ namespace clad {
     bool addToBlock(clang::Stmt* S, Stmts& block);
 
     /// Get a current scope.
-    clang::Scope* getCurrentScope() { return m_CurScope; }
+    /// FIXME: Remove the pointer-ref
+    // clang::Scope* getCurrentScope() { return m_Sema.getCurScope(); }
+    clang::Scope*& getCurrentScope();
+    void setCurrentScope(clang::Scope* S);
+    /// Returns the innermost enclosing file context which can be either a
+    /// namespace or the TU scope.
+    clang::Scope* getEnclosingNamespaceOrTUScope();
+
     /// Enters a new scope.
-    void beginScope(unsigned ScopeFlags) {
-      // FIXME: since Sema::CurScope is private, we cannot access it and have
-      // to use separate member variable m_CurScope. The only options to set
-      // CurScope of Sema seemt to be through Parser or ContextAndScopeRAII.
-      m_CurScope =
-          new clang::Scope(getCurrentScope(), ScopeFlags, m_Sema.Diags);
-    }
-    void endScope() {
-      // This will remove all the decls in the scope from the IdResolver.
-      m_Sema.ActOnPopScope(noLoc, m_CurScope);
-      auto oldScope = m_CurScope;
-      m_CurScope = oldScope->getParent();
-      delete oldScope;
-    }
+    void beginScope(unsigned ScopeFlags);
+    void endScope();
 
     /// A shorthand to simplify syntax for creation of new expressions.
     /// This function uses m_Sema.BuildUnOp internally to build unary
@@ -249,7 +279,23 @@ namespace clad {
                          clang::Expr* R, clang::SourceLocation OpLoc = noLoc);
 
     clang::Expr* BuildParens(clang::Expr* E);
-
+    /// Builds variable declaration to be used inside the derivative
+    /// body.
+    /// \param[in] Type The type of variable declaration to build.
+    /// \param[in] Identifier The identifier information for the variable
+    /// declaration.
+    /// \param[in] Init The initalization expression to assign to the variable
+    ///  declaration.
+    /// \param[in] DirectInit A check for if the initialization expression is a
+    /// C style initalization.
+    /// \param[in] TSI The type source information of the variable declaration.
+    /// \returns The newly built variable declaration.
+    clang::VarDecl*
+    BuildVarDecl(clang::QualType Type, clang::IdentifierInfo* Identifier,
+                 clang::Scope* scope, clang::Expr* Init = nullptr,
+                 bool DirectInit = false, clang::TypeSourceInfo* TSI = nullptr,
+                 clang::VarDecl::InitializationStyle IS =
+                     clang::VarDecl::InitializationStyle::CInit);
     /// Builds variable declaration to be used inside the derivative
     /// body.
     /// \param[in] Type The type of variable declaration to build.
@@ -283,6 +329,14 @@ namespace clad {
                  clang::TypeSourceInfo* TSI = nullptr,
                  clang::VarDecl::InitializationStyle IS =
                      clang::VarDecl::InitializationStyle::CInit);
+    /// Builds variable declaration to be used inside the derivative
+    /// body in the derivative function global scope.
+    clang::VarDecl*
+    BuildGlobalVarDecl(clang::QualType Type, llvm::StringRef prefix = "_t",
+                       clang::Expr* Init = nullptr, bool DirectInit = false,
+                       clang::TypeSourceInfo* TSI = nullptr,
+                       clang::VarDecl::InitializationStyle IS =
+                           clang::VarDecl::InitializationStyle::CInit);
     /// Creates a namespace declaration and enters its context. All subsequent
     /// Stmts are built inside that namespace, until
     /// m_Sema.PopDeclContextIsUsed.
@@ -292,14 +346,14 @@ namespace clad {
     /// \n Variable declaration cannot be added to code directly, instead we
     /// have to build a declaration staement.
     /// \param[in] D The declaration to build a declaration statement from.
-    /// \returns The declration statement expression corresponding to the input
+    /// \returns The declaration statement expression corresponding to the input
     /// variable declaration.
     clang::DeclStmt* BuildDeclStmt(clang::Decl* D);
     /// Wraps a set of declarations in a DeclStmt.
     /// \n This function is useful to wrap multiple variable declarations in one
     /// single declaration statement.
     /// \param[in] D The declarations to build a declaration statement from.
-    /// \returns The declration statemetn expression corresponding to the input
+    /// \returns The declaration statement expression corresponding to the input
     /// variable declaration.
     clang::DeclStmt* BuildDeclStmt(llvm::MutableArrayRef<clang::Decl*> DS);
 
@@ -308,8 +362,10 @@ namespace clad {
     /// declaration reference expressions. This function builds a declaration
     /// reference given a declaration.
     /// \param[in] D The declaration to build a DeclRefExpr for.
+    /// \param[in] SS The scope specifier for the declaration.
     /// \returns the DeclRefExpr for the given declaration.
-    clang::DeclRefExpr* BuildDeclRef(clang::DeclaratorDecl* D);
+    clang::DeclRefExpr* BuildDeclRef(clang::DeclaratorDecl* D,
+                                     const clang::CXXScopeSpec* SS = nullptr);
 
     /// Stores the result of an expression in a temporary variable (of the same
     /// type as is the result of the expression) and returns a reference to it.
@@ -333,15 +389,17 @@ namespace clad {
                              bool forceDeclCreation = false,
                              clang::VarDecl::InitializationStyle IS =
                                  clang::VarDecl::InitializationStyle::CInit);
+    /// For an expr E, decides if it is useful to store it in a temporary
+    /// variable and replace E's further usage by a reference to that variable
+    /// to avoid recomputation.
+    static bool UsefulToStore(clang::Expr* E);
     /// A flag for silencing warnings/errors output by diag function.
-    bool silenceDiags = false;
     /// Shorthand to issues a warning or error.
     template <std::size_t N>
     void diag(clang::DiagnosticsEngine::Level level, // Warning or Error
               clang::SourceLocation loc, const char (&format)[N],
               llvm::ArrayRef<llvm::StringRef> args = {}) {
-      if (!silenceDiags)
-        m_Builder.diag(level, loc, format, args);
+      m_Builder.diag(level, loc, format, args);
     }
 
     /// Creates unique identifier of the form "_nameBase<number>" that is
@@ -366,6 +424,13 @@ namespace clad {
     clang::Expr*
     BuildArraySubscript(clang::Expr* Base,
                         const llvm::SmallVectorImpl<clang::Expr*>& IS);
+
+    /// Build an array subscript expression with a given base expression and
+    /// one index.
+    clang::Expr* BuildArraySubscript(clang::Expr* Base, clang::Expr*& Idx) {
+      llvm::SmallVector<clang::Expr*, 1> IS = {Idx};
+      return BuildArraySubscript(Base, IS);
+    }
     /// Find namespace clad declaration.
     clang::NamespaceDecl* GetCladNamespace();
     /// Find declaration of clad::class templated type
@@ -399,6 +464,8 @@ namespace clad {
 
     clang::DeclRefExpr* GetCladTapePushDRE();
 
+    clang::Stmt* GetCladZeroInit(llvm::MutableArrayRef<clang::Expr*> args);
+
     /// Assigns the Init expression to VD after performing the necessary
     /// implicit conversion. This is required as clang doesn't add implicit
     /// conversions while assigning values to variables which are initialized
@@ -429,7 +496,7 @@ namespace clad {
     clang::Expr*
     BuildCallExprToMemFn(clang::Expr* Base, llvm::StringRef MemberFunctionName,
                          llvm::MutableArrayRef<clang::Expr*> ArgExprs,
-                         clang::ValueDecl* memberDecl = nullptr);
+                         clang::SourceLocation Loc = noLoc);
 
     /// Build a call to member function through this pointer.
     ///
@@ -438,10 +505,9 @@ namespace clad {
     /// \param[in] useRefQualifiedThisObj If true, then the `this` object is
     /// perfectly forwarded while calling member functions.
     /// \returns Built member function call expression
-    clang::Expr*
-    BuildCallExprToMemFn(clang::CXXMethodDecl* FD,
-                         llvm::MutableArrayRef<clang::Expr*> argExprs,
-                         bool useRefQualifiedThisObj = false);
+    clang::Expr* BuildCallExprToMemFn(
+        clang::CXXMethodDecl* FD, llvm::MutableArrayRef<clang::Expr*> argExprs,
+        bool useRefQualifiedThisObj = false, clang::SourceLocation Loc = noLoc);
 
     /// Build a call to a free function or member function through
     /// this pointer depending on whether the `FD` argument corresponds to a
@@ -455,7 +521,21 @@ namespace clad {
     clang::Expr*
     BuildCallExprToFunction(clang::FunctionDecl* FD,
                             llvm::MutableArrayRef<clang::Expr*> argExprs,
-                            bool useRefQualifiedThisObj = false);
+                            bool useRefQualifiedThisObj = false,
+                            const clang::CXXScopeSpec* SS = nullptr);
+
+    /// Build a call to templated free function inside the clad namespace.
+    ///
+    /// \param[in] name name of the function
+    /// \param[in] argExprs function arguments expressions
+    /// \param[in] templateArgs template arguments
+    /// \param[in] loc location of the call
+    /// \returns Built call expression
+    clang::Expr* BuildCallExprToCladFunction(
+        llvm::StringRef name, llvm::MutableArrayRef<clang::Expr*> argExprs,
+        llvm::ArrayRef<clang::TemplateArgument> templateArgs,
+        clang::SourceLocation loc);
+
     /// Find declaration of clad::array_ref templated type.
     clang::TemplateDecl* GetCladArrayRefDecl();
     /// Create clad::array_ref<T> type.
@@ -466,11 +546,19 @@ namespace clad {
     clang::TemplateDecl* GetCladArrayDecl();
     /// Create clad::array<T> type.
     clang::QualType GetCladArrayOfType(clang::QualType T);
+    /// Find declaration of clad::matrix templated type.
+    clang::TemplateDecl* GetCladMatrixDecl();
+    /// Create clad::matrix<T> type.
+    clang::QualType GetCladMatrixOfType(clang::QualType T);
+    /// Creates the expression clad::matrix<T>::identity(Args) for the given
+    /// type and args.
+    clang::Expr*
+    BuildIdentityMatrixExpr(clang::QualType T,
+                            llvm::MutableArrayRef<clang::Expr*> Args,
+                            clang::SourceLocation Loc);
     /// Creates the expression Base.size() for the given Base expr. The Base
     /// expr must be of clad::array_ref<T> type
     clang::Expr* BuildArrayRefSizeExpr(clang::Expr* Base);
-    /// Checks if the type is of clad::ValueAndPushforward<T,U> type
-    bool isCladValueAndPushforwardType(clang::QualType QT);
     /// Creates the expression Base.slice(Args) for the given Base expr and Args
     /// array. The Base expr must be of clad::array_ref<T> type
     clang::Expr*
@@ -479,7 +567,8 @@ namespace clad {
     clang::ParmVarDecl* CloneParmVarDecl(const clang::ParmVarDecl* PVD,
                                          clang::IdentifierInfo* II,
                                          bool pushOnScopeChains = false,
-                                         bool cloneDefaultArg = true);
+                                         bool cloneDefaultArg = true,
+                                         clang::SourceLocation Loc = noLoc);
     /// A function to get the single argument "forward_central_difference"
     /// call expression for the given arguments.
     ///
@@ -491,40 +580,25 @@ namespace clad {
     ///
     /// \returns The derivative function call.
     clang::Expr* GetSingleArgCentralDiffCall(
-        clang::Expr* targetFuncCall, clang::Expr* targetArg, 
-        unsigned targetPos, unsigned numArgs, 
-        llvm::SmallVectorImpl<clang::Expr*>& args);
-    /// A function to get the multi-argument "central_difference"
-    /// call expression for the given arguments.
-    ///
-    /// \param[in] targetFuncCall The function to get the derivative for.
-    /// \param[in] retType The return type of the target call expression.
-    /// \param[in] numArgs The total number of 'args'.
-    /// \param[in] NumericalDiffMultiArg The built statements to add to block
-    /// later.
-    /// \param[in] args All the arguments to the target function.
-    /// \param[in] outputArgs The output gradient arguments.
-    ///
-    /// \returns The derivative function call.
-    clang::Expr* GetMultiArgCentralDiffCall(
-        clang::Expr* targetFuncCall, clang::QualType retType, 
-        unsigned numArgs,
-        llvm::SmallVectorImpl<clang::Stmt*>& NumericalDiffMultiArg,
-        llvm::SmallVectorImpl<clang::Expr*>& args,
-        llvm::SmallVectorImpl<clang::Expr*>& outputArgs);
-    /// Emits diagnostic messages on differentiation (or lack thereof) for 
+        clang::Expr* targetFuncCall, clang::Expr* targetArg, unsigned targetPos,
+        unsigned numArgs, llvm::SmallVectorImpl<clang::Expr*>& args);
+
+    /// Emits diagnostic messages on differentiation (or lack thereof) for
     /// call expressions.
     ///
-    /// \param[in] \c funcName The name of the underlying function of the 
-    /// call expression.
-    /// \param[in] \c srcLoc Any associated source location information. 
-    /// \param[in] \c isDerived A flag to determine if differentiation of the
-    /// call expression was successful.
-    void CallExprDiffDiagnostics(llvm::StringRef funcName,
-                                 clang::SourceLocation srcLoc,
-                                 bool isDerived);
+    /// \param[in] \c FD - The function declaration.
+    /// \param[in] \c srcLoc Any associated source location information.
+    void CallExprDiffDiagnostics(const clang::FunctionDecl* FD,
+                                 clang::SourceLocation srcLoc);
 
     clang::QualType DetermineCladArrayValueType(clang::QualType T);
+
+    /// Returns clad::Identify template declaration.
+    clang::TemplateDecl* GetCladConstructorPushforwardTag();
+
+    /// Returns type clad::Identify<T>
+    clang::QualType GetCladConstructorPushforwardTagOfType(clang::QualType T);
+
   public:
     /// Rebuild a sequence of nested namespaces ending with DC.
     clang::NamespaceDecl* RebuildEnclosingNamespaces(clang::DeclContext* DC);
@@ -532,6 +606,46 @@ namespace clad {
     clang::Stmt* Clone(const clang::Stmt* S);
     /// A shorthand to simplify cloning of expressions.
     clang::Expr* Clone(const clang::Expr* E);
+    /// Cloning types is necessary since VariableArrayType
+    /// store a pointer to their size expression.
+    clang::QualType CloneType(clang::QualType T);
+
+    /// Computes effective derivative operands. It should be used when operands
+    /// might be of pointer types.
+    ///
+    /// In the trivial case, both operands are of non-pointer types, and the
+    /// effective derivative operands are `LDiff.getExpr_dx()` and
+    /// `RDiff.getExpr_dx()` respectively.
+    ///
+    /// Integers used in pointer arithmetic should be considered
+    /// non-differentiable entities. For example:
+    ///
+    /// ```
+    /// p + i;
+    /// ```
+    ///
+    /// Derived statement should be:
+    ///
+    /// ```
+    /// _d_p + i;
+    /// ```
+    ///
+    /// instead of:
+    ///
+    /// ```
+    /// _d_p + _d_i;
+    /// ```
+    ///
+    /// Therefore, effective derived expression of `i` is `i` instead of `_d_i`.
+    ///
+    /// This functions sets `derivedL` and `derivedR` arguments to effective
+    /// derived expressions.
+    void ComputeEffectiveDOperands(StmtDiff& LDiff, StmtDiff& RDiff,
+                                   clang::Expr*& derivedL,
+                                   clang::Expr*& derivedR);
+
+  private:
+    clang::TemplateDecl* m_CladConstructorPushforwardTag = nullptr;
   };
 } // end namespace clad
 
